﻿using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace NTemplates.DocumentStructure
{
    internal class CommonMethods
    {
        Parser _parser;

        internal CommonMethods(Parser p)
        {
            _parser = p;
        }

        /// <summary>
        /// Should move this to the "Control" classes hierarchy.
        /// </summary>
        /// <param name="OpenRegex"></param>
        /// <param name="textAux"></param>
        /// <returns></returns>
        internal string GetReplacementsForAllPlaceHolders(string textAux)
        {
            List<string> placeHolders = _parser.DataManager.GetAllPlaceHolders();
            foreach (string ph in placeHolders)
            {
                object value = _parser.DataManager.GetCurrentValueForPlaceHolder(ph);
                if (value != null)
                    textAux = textAux.Replace(ph, value.ToString());
            }                      
            return GetReplacementsForFunctions(textAux);
        }
        
        internal string GetReplacementsForFunctions(string text)
        {
            Regex funcs = CommonMethods.GetRegex(_parser.FunctionsRegexString);
            MatchCollection matches = funcs.Matches(text);
            string matchString;
            FunctionsEvaluator FEvaluator = new FunctionsEvaluator();
            
            foreach (Match m in matches)
            {
                matchString = m.ToString();

                if (Regex.Matches(matchString, Parser._dtfmt).Count > 0) //DateFormat
                {
                    //DateTimeFormat
                    text = text.Replace(m.ToString(), FEvaluator.Format(matchString, this._parser.DataManager, TemplateFunctions.DateFormat));
                } else
                    if (Regex.Matches(matchString, Parser._dbfmt).Count > 0) //DoubleFormat
                    {
                        //DoubleFormat
                        text = text.Replace(m.ToString(), FEvaluator.Format(matchString, this._parser.DataManager, TemplateFunctions.DoubleFormat));
                    }
            }
            return text;
        }
        
        internal static string prepare(string placeholder)
        {
            return Parser._d + placeholder + Parser._d;
        }

        internal static Regex GetRegex(string text)
        {
            Regex regex = new Regex(text, RegexOptions.IgnoreCase | RegexOptions.CultureInvariant | RegexOptions.Multiline | RegexOptions.Singleline);
            return regex;
        }
        
        internal static KeyWords GetKeyWord(Match match)
        {
            string sMatch = match.ToString();
            if (sMatch.Contains("SCAN(") && sMatch.Contains("FOR"))
            {
                return KeyWords.SCAN_FOR;
            }
            else
                if (sMatch.Contains("SCAN("))
                {
                    return KeyWords.SCAN;
                }
                else
                    if (sMatch.Contains("ENDSCAN"))
                    {
                        return KeyWords.ENDSCAN;
                    }
                    else
                        if (sMatch.Contains("ELSE"))
                            return KeyWords.ELSE;
                        else
                            if (sMatch.Contains("ENDIF"))
                                return KeyWords.ENDIF;
                            else
                                if (sMatch.Contains("IF"))
                                    return KeyWords.IF;
                                else
                                    return KeyWords.NONE;
        }
    }
}
